package com.sun.xml.internal.ws.streaming;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

public class XMLStreamReaderUtil
{
    private XMLStreamReaderUtil() {
    }
    
    public static void close(XMLStreamReader reader) {
        try {
            reader.close();
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static void readRest(XMLStreamReader reader) {
        try {
            while (reader.getEventType() != 8) {
                reader.next();
            }
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static int next(XMLStreamReader reader) {
        try {
            int readerEvent = reader.next();
            while (readerEvent != 8) {
                switch (readerEvent) {
                    case 1:
                    case 2:
                    case 3:
                    case 4:
                    case 12: {
                        return readerEvent;
                    }
                    default: {
                        readerEvent = reader.next();
                        continue;
                    }
                }
            }
            return readerEvent;
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static int nextElementContent(XMLStreamReader reader) {
        int state = nextContent(reader);
        if (state == 4) {
            throw new XMLStreamReaderException("xmlreader.unexpectedCharacterContent", new Object[] { reader.getText() });
        }
        return state;
    }
    
    public static void toNextTag(XMLStreamReader reader, QName name) {
        if (reader.getEventType() != 1 && reader.getEventType() != 2) {
            nextElementContent(reader);
        }
        if (reader.getEventType() == 2 && name.equals(reader.getName())) {
            nextElementContent(reader);
        }
    }
    
    public static String nextWhiteSpaceContent(XMLStreamReader reader) {
        next(reader);
        return currentWhiteSpaceContent(reader);
    }
    
    public static String currentWhiteSpaceContent(XMLStreamReader reader) {
        StringBuilder whiteSpaces = null;
        while (true) {
            switch (reader.getEventType()) {
                case 1:
                case 2:
                case 8: {
                    return (whiteSpaces == null) ? null : whiteSpaces.toString();
                }
                case 4: {
                    if (reader.isWhiteSpace()) {
                        if (whiteSpaces == null) {
                            whiteSpaces = new StringBuilder();
                        }
                        whiteSpaces.append(reader.getText());
                        break;
                    }
                    else {
                        throw new XMLStreamReaderException("xmlreader.unexpectedCharacterContent", new Object[] { reader.getText() });
                    }
                }
            }
            next(reader);
        }
    }
    
    public static int nextContent(XMLStreamReader reader) {
        while (true) {
            int state = next(reader);
            switch (state) {
                case 1:
                case 2:
                case 8: {
                    return state;
                }
                case 4: {
                    if (!reader.isWhiteSpace()) {
                        return 4;
                    }
                    continue;
                }
            }
        }
    }
    
    public static void skipElement(XMLStreamReader reader) {
        assert reader.getEventType() == 1;
        skipTags(reader, true);
        assert reader.getEventType() == 2;
        return;
    }
    
    public static void skipSiblings(XMLStreamReader reader, QName parent) {
        skipTags(reader, reader.getName().equals(parent));
        assert reader.getEventType() == 2;
        return;
    }
    
    private static void skipTags(XMLStreamReader reader, boolean exitCondition) {
        try {
            int tags = 0;
            int state;
            while ((state = reader.next()) != 8) {
                if (state == 1) {
                    ++tags;
                }
                else {
                    if (state != 2) {
                        continue;
                    }
                    if (tags == 0 && exitCondition) {
                        return;
                    }
                    --tags;
                }
            }
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static String getElementText(XMLStreamReader reader) {
        try {
            return reader.getElementText();
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static QName getElementQName(XMLStreamReader reader) {
        try {
            String text = reader.getElementText().trim();
            String prefix = text.substring(0, text.indexOf(58));
            String namespaceURI = reader.getNamespaceContext().getNamespaceURI(prefix);
            if (namespaceURI == null) {
                namespaceURI = "";
            }
            String localPart = text.substring(text.indexOf(58) + 1, text.length());
            return new QName(namespaceURI, localPart);
        }
        catch (XMLStreamException e) {
            throw wrapException(e);
        }
    }
    
    public static Attributes getAttributes(XMLStreamReader reader) {
        return (reader.getEventType() == 1 || reader.getEventType() == 10) ? new AttributesImpl(reader) : null;
    }
    
    public static void verifyReaderState(XMLStreamReader reader, int expectedState) {
        int state = reader.getEventType();
        if (state != expectedState) {
            throw new XMLStreamReaderException("xmlreader.unexpectedState", new Object[] { getStateName(expectedState), getStateName(state) });
        }
        return;
    }
    
    public static void verifyTag(XMLStreamReader reader, String namespaceURI, String localName) {
        if (!localName.equals(reader.getLocalName()) || !namespaceURI.equals(reader.getNamespaceURI())) {
            throw new XMLStreamReaderException("xmlreader.unexpectedState.tag", new Object[] { "{" + namespaceURI + "}" + localName, "{" + reader.getNamespaceURI() + "}" + reader.getLocalName() });
        }
        return;
    }
    
    public static void verifyTag(XMLStreamReader reader, QName name) {
        verifyTag(reader, name.getNamespaceURI(), name.getLocalPart());
    }
    
    public static String getStateName(XMLStreamReader reader) {
        return getStateName(reader.getEventType());
    }
    
    public static String getStateName(int state) {
        switch (state) {
            case 10: {
                return "ATTRIBUTE";
            }
            case 12: {
                return "CDATA";
            }
            case 4: {
                return "CHARACTERS";
            }
            case 5: {
                return "COMMENT";
            }
            case 11: {
                return "DTD";
            }
            case 8: {
                return "END_DOCUMENT";
            }
            case 2: {
                return "END_ELEMENT";
            }
            case 15: {
                return "ENTITY_DECLARATION";
            }
            case 9: {
                return "ENTITY_REFERENCE";
            }
            case 13: {
                return "NAMESPACE";
            }
            case 14: {
                return "NOTATION_DECLARATION";
            }
            case 3: {
                return "PROCESSING_INSTRUCTION";
            }
            case 6: {
                return "SPACE";
            }
            case 7: {
                return "START_DOCUMENT";
            }
            case 1: {
                return "START_ELEMENT";
            }
            default: {
                return "UNKNOWN";
            }
        }
    }
    
    private static XMLStreamReaderException wrapException(XMLStreamException e) {
        return new XMLStreamReaderException("xmlreader.ioException", new Object[] { e });
    }
    
    public static class AttributesImpl implements Attributes
    {
        static final String XMLNS_NAMESPACE_URI = "http://www.w3.org/2000/xmlns/";
        XMLStreamReaderUtil.AttributesImpl.AttributeInfo[] atInfos;
        
        public AttributesImpl(XMLStreamReader reader) {
            if (reader == null) {
                this.atInfos = new XMLStreamReaderUtil.AttributesImpl.AttributeInfo[0];
            }
            else {
                int index = 0;
                int namespaceCount = reader.getNamespaceCount();
                int attributeCount = reader.getAttributeCount();
                this.atInfos = new XMLStreamReaderUtil.AttributesImpl.AttributeInfo[namespaceCount + attributeCount];
                String namespacePrefix;
                for (int i = 0; i < namespaceCount; this.atInfos[index++] = new XMLStreamReaderUtil.AttributesImpl.AttributeInfo(new QName("http://www.w3.org/2000/xmlns/", namespacePrefix, "xmlns"), reader.getNamespaceURI(i)), ++i) {
                    namespacePrefix = reader.getNamespacePrefix(i);
                    if (namespacePrefix == null) {
                        namespacePrefix = "";
                    }
                }
                for (int i = 0; i < attributeCount; ++i) {
                    this.atInfos[index++] = new XMLStreamReaderUtil.AttributesImpl.AttributeInfo(reader.getAttributeName(i), reader.getAttributeValue(i));
                }
            }
        }
        
        @Override
        public int getLength() {
            return this.atInfos.length;
        }
        
        @Override
        public String getLocalName(int index) {
            if (index >= 0 && index < this.atInfos.length) {
                return this.atInfos[index].getLocalName();
            }
            return null;
        }
        
        @Override
        public QName getName(int index) {
            if (index >= 0 && index < this.atInfos.length) {
                return this.atInfos[index].getName();
            }
            return null;
        }
        
        @Override
        public String getPrefix(int index) {
            if (index >= 0 && index < this.atInfos.length) {
                return this.atInfos[index].getName().getPrefix();
            }
            return null;
        }
        
        @Override
        public String getURI(int index) {
            if (index >= 0 && index < this.atInfos.length) {
                return this.atInfos[index].getName().getNamespaceURI();
            }
            return null;
        }
        
        @Override
        public String getValue(int index) {
            if (index >= 0 && index < this.atInfos.length) {
                return this.atInfos[index].getValue();
            }
            return null;
        }
        
        @Override
        public String getValue(QName name) {
            int index = this.getIndex(name);
            if (index != -1) {
                return this.atInfos[index].getValue();
            }
            return null;
        }
        
        @Override
        public String getValue(String localName) {
            int index = this.getIndex(localName);
            if (index != -1) {
                return this.atInfos[index].getValue();
            }
            return null;
        }
        
        @Override
        public String getValue(String uri, String localName) {
            int index = this.getIndex(uri, localName);
            if (index != -1) {
                return this.atInfos[index].getValue();
            }
            return null;
        }
        
        @Override
        public boolean isNamespaceDeclaration(int index) {
            return index >= 0 && index < this.atInfos.length && this.atInfos[index].isNamespaceDeclaration();
        }
        
        @Override
        public int getIndex(QName name) {
            int i = 0;
            while (i < this.atInfos.length) {
                if (this.atInfos[i].getName().equals(name)) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
        
        @Override
        public int getIndex(String localName) {
            int i = 0;
            while (i < this.atInfos.length) {
                if (this.atInfos[i].getName().getLocalPart().equals(localName)) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
        
        @Override
        public int getIndex(String uri, String localName) {
            int i = 0;
            while (i < this.atInfos.length) {
                QName qName = this.atInfos[i].getName();
                if (qName.getNamespaceURI().equals(uri) && qName.getLocalPart().equals(localName)) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
        
        static class AttributeInfo
        {
            private QName name;
            private String value;
            
            public AttributeInfo(QName name, String value) {
                this.name = name;
                if (value == null) {
                    this.value = "";
                }
                else {
                    this.value = value;
                }
            }
            
            QName getName() {
                return this.name;
            }
            
            String getValue() {
                return this.value;
            }
            
            String getLocalName() {
                if (this.isNamespaceDeclaration()) {
                    if (this.name.getLocalPart().equals("")) {
                        return "xmlns";
                    }
                    return "xmlns:" + this.name.getLocalPart();
                }
                else {
                    return this.name.getLocalPart();
                }
            }
            
            boolean isNamespaceDeclaration() {
                return this.name.getNamespaceURI() == "http://www.w3.org/2000/xmlns/";
            }
        }
    }
}
